/*************************************************************************
Author:   	$Author: dennis $
File:     	$HeadURL: file:///home/dennis/svn-store/avr-source/mptry-fat32/sd_mmc.c $
Date:  		$Date: 2008-10-16 19:14:26 +0200 (Do, 16. Okt 2008) $
Revision: 	$Revision: 164 $
Id: 		$Id: sd_mmc.c 164 2008-10-16 17:14:26Z dennis $
Licence:	GNU General Public License
 *************************************************************************/
/*
 * Copyright (c) 2006-2008 by Roland Riegel <feedback@roland-riegel.de>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1, both as
 * published by the Free Software Foundation.
 */

#include <string.h>
#include <avr/io.h>
#include "sd_mmc.h"

#include "uart.h"
#include "spi.h"

/**
 * \addtogroup sd_mmc MMC/SD card raw access
 *
 * This module implements read and write access to MMC and
 * SD cards. It serves as a low-level driver for the higher
 * level modules such as partition and file system access.
 *
 * @{
 */
/**
 * \file
 * MMC/SD access implementation (license: GPLv2 or LGPLv2.1)
 *
 * \author Roland Riegel; adapted to SPI layer Dennis Dingeldein
 */

/**
 * \addtogroup sd_config MMC/SD configuration
 * Preprocessor defines to configure the MMC/SD support.
 */

/**
 * @}
 */

/* commands available in SPI mode */

/* CMD0: response R1 */
#define CMD_GO_IDLE_STATE (0x40+0x00)
/* CMD1: response R1 */
#define CMD_SEND_OP_COND (0x40+0x01)
/* CMD8: response R7 */
#define CMD_SEND_IF_COND (0x40+0x08)
/* CMD9: response R1 */
#define CMD_SEND_CSD (0x40+0x09)
/* CMD10: response R1 */
#define CMD_SEND_CID (0x40+0x0a)
/* CMD13: response R2 */
#define CMD_SEND_STATUS (0x40+0x0d)
/* CMD16: arg0[31:0]: block length, response R1 */
#define CMD_SET_BLOCKLEN (0x40+0x10)

#define CMD55	(0x40+55)	/* APP_CMD */
#define ACMD41	(0xC0+41)	/* SEND_OP_COND (SDC) */
#define CMD58	(0x40+58) /* READ_OCR */

/* command responses */
/* R1: size 1 byte */
#define R1_IDLE_STATE 0


/* private helper functions */
static uint8_t sd_send_command_r1(uint8_t command, uint32_t arg);
//static uint16_t sd_send_command_r2(uint8_t command, uint32_t arg);


extern uint8_t CardType;
/**
 * \ingroup sd_mmc
 * Initializes memory card communication.
 *
 * \returns 1 on failure, 0 on success.
 */
uint8_t sd_init()
{
	uint8_t b;

	/* enable outputs for MOSI, SCK, SS, input for MISO */
	configure_pin_mosi();
	configure_pin_sck();
	configure_pin_ss();
	configure_pin_miso();

	/* this is the CS-line for SD/MMC card -initialize as output */
	DDRA |= (1 << DDA0);

	/* initialization procedure */

	unselect_card();

	/* spi init */
	/* the initialization must be done with 400Khz clock frequency (or lower) */
	b = spi_init();

	/* card needs minimum 74 cycles minimum to start up, read arbitrary bytes in to create cycles */
	for(uint8_t i = 0; i < 10; ++i) {
		/* create 8 clock cycles */
		spi_read_byte();
	}

	/* select card */
	select_card();

	/* send card initialization command */
	uint8_t response;
	for(uint16_t i = 0; ; ++i) {
		response = sd_send_command_r1(CMD_GO_IDLE_STATE, 0);
		/*expected answer is 0x01 */
		if(response == (1 << R1_IDLE_STATE))
			break;
		uart_putc_hex(response);
		uart_putc('.');
		if(i == 0x1ff) {

			/* timeout value reached - got no valid response */
			unselect_card();
			return 1;
		}
	}
	uart_puts("Init request accepted by card\r\n");

	// check for card type inserted:
	// MMC: original and oldest standard
	// SDSC: SD-Card, Standard Capacity, SD cards with less than 2G and usually Fat16 filesystem
	// SDHC: SD-Card, High Capacity, SD cards with more that 2G and usually Fat32
	// Note: Some companies have produced SDSC with more than 2G, this is non standard.

	// after GO_IDLE_STATE and before finishing initialization
	// it is allowed to send SEND_IF_COND
	// this only succeeds for SDHC, so we try it out:
	response = sd_send_command_r1(CMD_SEND_IF_COND, 0x1AA);
	if (response == 0x01) {
		uint8_t n, ocr[4];
		uart_puts("SDHC\n");
		for (n = 0; n < 4; n++)
			ocr[n] = spi_read_byte(); /* Get trailing return value of R7 resp */
		if (ocr[2] == 0x01 && ocr[3] == 0xAA) {
			/* The card can work at vdd range of 2.7-3.6V */
			uart_puts("voltage range of card ok\n\r");
			/* Wait for leaving idle state (ACMD41 with HCS bit) */
			// TODO: while loop with imeout
			while (sd_send_command_r1(ACMD41, 1UL << 30))
				;	/* Wait for leaving idle state (ACMD41 with HCS bit) */
			if (sd_send_command_r1(CMD58, 0) == 0) {
				/* Check CCS bit in the OCR */
				uart_puts("cmd58 ok\n\r");
				for (n = 0; n < 4; n++)
					ocr[n] = spi_read_byte();
				// TODO global type value, fill also for sd mmc
				CardType = (ocr[0] & 0x40) ? 12 : 4;
			}
		}
	} else {
		//uart_puts("NO SDHC\r\n");
		// MMC does not understand ACMD41, but SDSC does, so we try it out:
		response = sd_send_command_r1(ACMD41, 0x1AA);
		if (response <= 0x01) {
			// command was understood, it is a SDSC
			uart_puts("SDSC\r\n");
			// set correct finsihing command
			CardType = 1;
			b = ACMD41;
		} else {
			// command was not understood, so it must be a MMC.
			uart_puts("MMC\r\n");
			// set correct finsihing command
			CardType = 2;
			b = CMD_SEND_OP_COND;
		}
		/* wait for card to finish initialization */
		for(uint16_t i = 0; ; ++i) {
			response = sd_send_command_r1(b, 0);
			/* expected answer is 0x00 */
			if(!(response & (1 << R1_IDLE_STATE)))
				break;
			uart_putc('.');

			if(i == 0x7fff) {
				/* timeout value reached - got no valid response */
				unselect_card();
				return 1;
			}
		}
		for(uint16_t i = 0; ; ++i) {
			response = sd_send_command_r1(CMD_SET_BLOCKLEN, 512);
			//uart_putc_hex(response);
			/* expected answer is 0x00 */
			if(!response)
				break;
			uart_putc('.');

			if(i == 0x1ff) {
				/* timeout value reached - got no valid response */
				uart_puts("\n\rError setting block size\n\r");
				unselect_card();
				return 1;
			}
		}
		uart_puts("\n\rBlock size set.\n\r");
	}

	uart_puts("\r\nCard finished init\r\n");


	/* deaddress card */
	unselect_card();

	/* card initialization is complete */

	/* switch now to highest spi frequency possible for fast transfers */
	b = spi_set_maxspeed();

	return 0;
}


/**
 * \ingroup sd_mmc
 * Send a command to the memory card which responses with a R1 response.
 *
 * \param[in] command The command to send.
 * \param[in] arg The argument for command.
 * \returns The command answer.
 */
uint8_t sd_send_command_r1(uint8_t command, uint32_t arg)
{
	uint8_t response, n=0xff;

	/* for app commands, do a single rcursive call */
	if (command & 0x80) {	/* ACMD<n> is the command sequence of CMD55-CMD<n> */
		command &= 0x7F;
		response = sd_send_command_r1(CMD55, 0);
		if (response > 1) return response;
	}

	/* wait some clock cycles */
	spi_read_byte();

	/* send command via SPI */
	//spi_write_byte(0x40 | command);
	spi_write_byte(command);
	spi_write_byte((arg >> 24) & 0xff);
	spi_write_byte((arg >> 16) & 0xff);
	spi_write_byte((arg >> 8) & 0xff);
	spi_write_byte((arg >> 0) & 0xff);
	//spi_write_byte(command == CMD_GO_IDLE_STATE ? 0x95 : 0xff);
	if (command == CMD_GO_IDLE_STATE) n = 0x95; /* Valid CRC for CMD0(0) */
	if (command == CMD_SEND_IF_COND) n = 0x87;  /* Valid CRC for CMD8(0x1AA) */
	spi_write_byte(n);
	/* receive response */
	for(uint8_t i = 0; i < 10; ++i)
	{
		response = spi_read_byte();
		if(response != 0xff)
			break;
	}

	return response;
}

