/*************************************************************************
Author:   	$Author: dennis $
File:     	$HeadURL: file:///home/dennis/svn-store/avr-source/mptry-fat32/spi.c $
Date:  		$Date: 2008-10-13 19:35:16 +0200 (Mo, 13. Okt 2008) $
Revision: 	$Revision: 150 $
Id: 		$Id: spi.c 150 2008-10-13 17:35:16Z dennis $
Licence:	GNU General Public License
 *************************************************************************/

#include "spi.h"

#include <avr/io.h>

/**
 * \ingroup spi
 * Init spi communication registers. Speed is set up initially to f_OSC/64.
 *
 * \returns 0 on success.
 */
uint8_t spi_init() {
    /* enable outputs for MOSI, SCK, SS, input for MISO */
    configure_pin_mosi();
    configure_pin_sck();
    configure_pin_ss();
    configure_pin_miso();

	/* initialize SPI with lowest frequency; max. 400kHz during identification mode of card */
	/*| SPR1 | SPR0 | SCK-Frequenz     |
	  ----------------------------------
	  |   0  |   0  | clk/4            |
	  |   0  |   1  | clk/16           |<--  max for my experimental board with long lines
	  |   1  |   0  | clk/64           |
	  |   1  |   1  | clk/128          |*/

    SPCR = (0 << SPIE) | /* SPI Interrupt Enable */
           (1 << SPE)  | /* SPI Enable */
           (0 << DORD) | /* Data Order: MSB first */
           (1 << MSTR) | /* Master mode */
           (0 << CPOL) | /* Clock Polarity: SCK low when idle */
           (0 << CPHA) | /* Clock Phase: sample on rising SCK edge */
           (1 << SPR1) | /* Clock Frequency: see table above */
           (1 << SPR0);
    SPSR &= ~(1 << SPI2X); /* No doubled clock frequency */
    return 0;
}

/**
 * \ingroup spi
 * Set spi transfer speed to maximum for this chip.
 *
 * \returns 0 on success.
 */uint8_t spi_set_maxspeed() {
    /* switch to highest SPI frequency possible */
    //SPCR &= ~((1 << SPR1) | ~(1 << SPR0)); /* Clock Frequency: f_OSC / 4 */
	SPCR &= ~(1 << SPR1);
	SPCR |=  (1 << SPR0); /* Clock Frequency: f_OSC / 16 now */
    SPSR |= (1 << SPI2X); /* Doubled Clock Frequency --> f_OSC / 8 */
    return 0;
}

/**
 * \ingroup spi
 * Reads a byte from spi. Before the byte is read in, a dummy byte (0xff) is written to spi
 * to create clock cycles to be used (for syncronizing) by slave.
 *
 * \returns the byte read.
 */
uint8_t spi_read_byte() {
    /* send dummy data for receiving some */
    SPDR = 0xff;
    /* wait for dummy byte to be shifted out */
    while(!(SPSR & (1 << SPIF)))
    	;
    /* wait for byte to be shifted in */
    SPSR &= ~(1 << SPIF);

    return SPDR;
}

/**
 * \ingroup spi
 * Write a byte to spi.
 *
 * \param[in] byte the byte to write to spi
 * \returns byte written on success.
 */
uint8_t spi_write_byte( int8_t byte ) {
	SPDR = byte;
	/* wait for byte to be shifted out */
	while(!(SPSR & (1 << SPIF)))
		;
	SPSR &= ~(1 << SPIF);
	return SPDR;
}
