/*************************************************************************
Author:   	$Author: dennis $
File:     	$HeadURL: file:///home/dennis/svn-store/avr-source/mptry-1/spi.c $
Date:  		$Date: 2008-09-06 10:51:00 +0200 (Sa, 06 Sep 2008) $ 
Revision: 	$Revision: 94 $ 
Id: 		$Id: spi.c 94 2008-09-06 08:51:00Z dennis $ 
Licence:	GNU General Public License
 *************************************************************************/

#include "vs1011.h"
#include "spi.h"

#include "delay.h"
#include <util/delay.h>

/* SCI read write values */
#define VS_WRITE 0x02
#define VS_READ  0x03

/* SCI register addresses */
#define SCI_MODE 			0x00
#define SCI_STATUS 		0x01
#define SCI_BASS 			0x02
#define SCI_CLOCKF 		0x03
#define SCI_DECODE_TIME 	0x04
#define SCI_HDAT0			0x08
#define SCI_HDAT1			0x09
#define SCI_VOL			0x0b

/* SCI_MODE register bit names */
#define SM_DIFF 		0
#define SM_LAYER12 	1
#define SM_RESET 		2
#define SM_OUTOFWAV 	3
#define SM_SETTOZERO1 4
#define SM_TESTS		5
#define SM_STREAM		6
#define SM_SETTOZERO2	7
#define SM_DACT		8
#define SM_SDIORD		9
#define SM_SDISHARE	10
#define SM_SDINEW		11
#define SM_SETTOZERO3	12
#define SM_SETTOZERO4	13


/**
 * \ingroup vs1011
 * Init vs1011 communication ports.
 *
 * \returns 0 on success.
 */
uint8_t vs1011_port_init( void ) {
	DDRA &= ~(1 << VS1011_DREQ); // input 
	DDRA |= (1 << VS1011_XCS); // output
	DDRA |= (1 << VS1011_XRESET); // output
#ifdef NON_SHARED
	// for non shared, XDCS is also uses 
	DDRA |= (1 << VS1011_XDCS);
#endif
	return 0;
}

static void _vs1011_init( void ) {
	// set clock value
	vs1011_xcs_select();
	spi_write_byte(VS_WRITE);
	spi_write_byte(SCI_CLOCKF);
	spi_write_byte(0x98); // 12.288 Mhz and clock doubler
	spi_write_byte(0x00);
	vs1011_xcs_deselect();	

	// set operating mode
    vs1011_xcs_select();
	spi_write_byte(VS_WRITE);
	spi_write_byte(SCI_MODE);
	//spi_write_byte(0x0c); // SM_SDINEW + SM_SDISHARE
	spi_write_byte(0x08); // SM_SDINEW 
	spi_write_byte(0x00); // 
	vs1011_xcs_deselect(); // end SCI cycle    
}

/**
 * \ingroup vs1011
 * SW Reset for vs1011.
 *
 * \returns 0 on success.
 */
uint8_t vs1011_sw_reset( void ) {

	//uart_puts("before sw init vs\n\r");
	vs1011_xcs_select();
	spi_write_byte(VS_WRITE);
	spi_write_byte(SCI_MODE); // address mode register
	spi_write_byte(0x00);
	spi_write_byte(1 << SM_RESET);
	vs1011_xcs_deselect();
	
	// wait until DREQ goes high
	loop_until_bit_is_set(PINA,VS1011_DREQ);
	//uart_puts("after sw init vs\n\r");

	_vs1011_init();

	return 0;
}

/**
 * \ingroup vs1011
 * HW Reset for vs1011.
 *
 * \returns 0 on success.
 */
uint8_t vs1011_init( void ) {

	// clear xreset pin -> hardware reset
	PORTA &= ~(1 << VS1011_XRESET);
	// set xreset pin -> vs1011 will go out of hardware reset state
	PORTA |= (1 << VS1011_XRESET);
	
	// wait until DREQ goes high
	loop_until_bit_is_set(PINA,VS1011_DREQ);

	_vs1011_init();

	return 0;
}

/**
 * \ingroup vs1011
 * Select vs1011 in SCI mode (xCS)
 *
 */
void vs1011_xcs_select( void ) {
	PORTA &= ~(1 << VS1011_XCS);
}

/**
 * \ingroup vs1011
 * Deselect vs1011 in SCI mode (xCS)
 *
 */
void vs1011_xcs_deselect( void ) {
	PORTA |= (1 << VS1011_XCS);
}

/**
 * \ingroup vs1011
 * Select vs1011 in SDI mode (xDCS).
 *
 */
void vs1011_xdcs_select( void ) {
	PORTA &= ~(1 << VS1011_XDCS); // activate XDCS
}

/**
 * \ingroup vs1011
 * Deselect vs1011 in SDI mode (xDCS).
 *
 */
void vs1011_xdcs_deselect( void ) {
	PORTA |= (1 << VS1011_XDCS);
}

/**
 * \ingroup vs1011
 * set volume for vs1011.
 * 
 * \param[in] v - volume value. 0x00 is loudest, 0xff maximum muted
 * \returns 0 on success.
 */
uint8_t vs1011_dim_volume( uint8_t v ) {
	// write volume value ; the lower the louder, 0xff is quiet!
	vs1011_xcs_select();
	spi_write_byte(VS_WRITE);
	spi_write_byte(SCI_VOL);
	spi_write_byte(v); // left 
	spi_write_byte(v); // right
	vs1011_xcs_deselect();
	
	return 0;
}

uint8_t vs1011_test_vol( void ) {
	//uart_puts("before vol vs\n\r");
	for (;;) {
		// test routine
		vs1011_xcs_select();
		spi_write_byte(VS_WRITE);
		spi_write_byte(SCI_VOL);
		spi_write_byte(0x00);
		spi_write_byte(0x00);
		vs1011_xcs_deselect();
		_delay_loop_2(1000);

		vs1011_xcs_select();
		spi_write_byte(VS_WRITE);
		spi_write_byte(SCI_VOL);
		spi_write_byte(0xff);
		spi_write_byte(0xff);
		vs1011_xcs_deselect();
		_delay_loop_2(1000);
	}
	return 0;
}


uint8_t vs1011_test_vol_rw( void ) {
	uint8_t b;
	
	// write volume value 0xa2, 0xf5
	vs1011_xcs_select();
	spi_write_byte(VS_WRITE);
	spi_write_byte(SCI_VOL);
	spi_write_byte(0xa2);
	spi_write_byte(0xf5);
	vs1011_xcs_deselect();

	// try to read back volume value; should be 0xa2, 0xf5
	vs1011_xcs_select();
	spi_write_byte(VS_READ);
	spi_write_byte(SCI_VOL);
	b = spi_write_byte(0x00);
	uart_putc_hex(b);
	b = spi_write_byte(0x00);
	uart_putc_hex(b);	
	vs1011_xcs_deselect();
	return 0;
}

uint8_t vs1011_test_sine_on_off( void ) {
	vs1011_xcs_select(); // start SCI cycle
	spi_write_byte(VS_WRITE);
	spi_write_byte(SCI_MODE);
#ifdef NON_SHARED
	spi_write_byte(0x08); // SD_SDINEW
	spi_write_byte(1<<SM_TESTS); // SM_TESTS 	
	vs1011_xcs_deselect(); // end SCI cycle
	PORTA &= ~(1 << VS1011_XDCS); // activate XDCS
#else

	spi_write_byte(0x0c); // SM_SDINEW + SM_SDISHARE
	spi_write_byte(1<<SM_TESTS); // SM_TESTS 	
	vs1011_xcs_deselect(); // end SCI cycle
#endif
	
	for (;;) {
		//uart_puts("sine on\n\r");
		spi_write_byte(0x53);
		spi_write_byte(0xef);
		spi_write_byte(0x6e); 
		spi_write_byte(0x44); 	
		spi_write_byte(0x00);
		spi_write_byte(0x00);
		spi_write_byte(0x00);
		spi_write_byte(0x00);
		dd_delay(10);	
		
		//uart_puts("sine off\n\r");
		spi_write_byte(0x45);
		spi_write_byte(0x78);
		spi_write_byte(0x69); 
		spi_write_byte(0x74); 	
		spi_write_byte(0x00);
		spi_write_byte(0x00);
		spi_write_byte(0x00);
		spi_write_byte(0x00);
		dd_delay(10);	
	}
	return 0;
}
